/**
 * Classe Renderer, junto com a classe Janela, mostra um exemplo de 
 * como trabalhar com modelagem em OpenGL.
 * 
 * @author Marcelo Cohen
 * @version 1.0
 */

import java.awt.event.*; 
import javax.media.opengl.*;
import javax.media.opengl.glu.*;
import com.sun.opengl.util.*;

public class Renderer extends MouseAdapter implements GLEventListener, KeyListener
{
    // Atributos
    private GL gl;
    private GLU glu;
    private GLUT glut;
    private GLAutoDrawable glDrawable;
    private double fAspect;
    
    /**
     * Construtor da classe Renderer
     */
    public Renderer()
    {
        // Inicializa o valor para correção de aspecto   
        fAspect = 1;
    }

    /**
     * Método definido na interface GLEventListener e chamado pelo objeto no qual será feito o desenho
     * logo após a inicialização do contexto OpenGL. 
     */    
    public void init(GLAutoDrawable drawable)
    {
        glDrawable = drawable;
        gl = drawable.getGL();
        // glu = drawable.getGLU();       
        glu = new GLU();
        glut = new GLUT();

        drawable.setGL(new DebugGL(gl));        

        gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
        
        // Define a janela de visualização 2D
        gl.glMatrixMode(GL.GL_PROJECTION);
        glu.gluOrtho2D(-10,10,-10,10);
        gl.glMatrixMode(GL.GL_MODELVIEW);
    }

    /**
     * Desenha uma grade e os eixos cartesianos
     */
    public void desenhaGrade()
    {
        gl.glColor3f(0.8f,0.8f,0.8f);
        gl.glLineWidth(1.0f);
        gl.glBegin(GL.GL_LINES);
        for(float x=-10; x<=10; x++)
        {
            gl.glVertex2f(-10,x);
            gl.glVertex2f( 10,x);
            gl.glVertex2f(x,-10);
            gl.glVertex2f(x, 10);
        }
        gl.glEnd();
        gl.glColor3f(0f,0f,0f);
        gl.glLineWidth(3.0f);
        gl.glBegin(GL.GL_LINES);
        gl.glVertex2f(-10,0);
        gl.glVertex2f( 10,0);
        gl.glVertex2f(0,-10);
        gl.glVertex2f(0, 10);
        gl.glEnd();
    }

    /**
     * Método definido na interface GLEventListener e chamado pelo objeto no qual será feito o desenho
     * para começar a fazer o desenho OpenGL pelo cliente.
     */  
    public void display(GLAutoDrawable drawable)
    {
        gl.glClear(GL.GL_COLOR_BUFFER_BIT);
        gl.glLoadIdentity();    

        desenhaGrade();
    }

    /**
     * Método definido na interface GLEventListener e chamado pelo objeto no qual será feito o desenho
     * depois que a janela foi redimensionada.
     */  
    public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height)
    {
        gl.glViewport(0, 0, width, height);
        fAspect = (float)width/(float)height;      
    }

    /**
     * Método definido na interface GLEventListener e chamado pelo objeto no qual será feito o desenho
     * quando o modo de exibição ou o dispositivo de exibição associado foi alterado.
     */  
    public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) { }

    /**
     * Método da classe MouseAdapter que está sendo sobrescrito para gerenciar os 
     * eventos de clique de mouse, de maneira que seja feito zoom in e zoom out.
     */  
    public void mouseClicked(MouseEvent e)
    {
        if (e.getButton() == MouseEvent.BUTTON1) // botão 1
        { }
        if (e.getButton() == MouseEvent.BUTTON3) // botão 2
        { }
        glDrawable.display();
    }

    /**
     * Método definido na interface KeyListener que está sendo implementado para, 
     * de acordo com as teclas pressionadas, permitir alterações sobre os objetos
         * ou área de desenho.
     */        
    public void keyPressed(KeyEvent e)
    {
        // F1
        if(e.getKeyCode()==KeyEvent.VK_F1)
        {
        }
        // F2
        else if(e.getKeyCode()==KeyEvent.VK_F2)
        {
        }

        switch (e.getKeyCode())
        {
            case KeyEvent.VK_1:         System.out.println("Pressionou 1");
                                        break;
            case KeyEvent.VK_2:         System.out.println("Pressionou 2");
                                        break;     
            case KeyEvent.VK_ESCAPE:    System.exit(0);
                                        break;
        }  
        glDrawable.display();
    }

    /**
     * Método definido na interface KeyListener.
     */      
    public void keyTyped(KeyEvent e) { }

    /**
     * Método definido na interface KeyListener.
     */       
    public void keyReleased(KeyEvent e) { }
}

